@@ -56,14 +56,14 @@ otherwise.
 ### XXX remove this list and move it into selfupdate, somehow..
 ### this is dual administration
 {   my $query_list = {
-        'File::Fetch'   => '0.13_02',
-        'YAML::Tiny'    => '0.0',
-        'File::Temp'    => '0.0',
+        'File::Fetch'          => '0.13_02',
+        'Parse::CPAN::Meta'    => '0.0',
+        'File::Temp'           => '0.0',
     };
 
     my $send_list = {
         %$query_list,
-        'Test::Reporter' => '1.34',
+        'Test::Reporter' => '1.54',
     };
 
     sub _have_query_report_modules {
@@ -196,7 +196,7 @@ sub _query_report {
         do { local $/; <$fh> };
     };
 
-    my ($aref) = eval { YAML::Tiny::Load( $res ) };
+    my ($aref) = eval { Parse::CPAN::Meta::Load( $res ) };
 
     if( $@ ) {
         error(loc("Error reading result: %1", $@));
@@ -214,8 +214,8 @@ sub _query_report {
         $href->{'details'}  = $details;
         
         ### backwards compatibility :(
-        $href->{'dist'}     = delete $href->{'distversion'};
-        $href->{'grade'}    = delete $href->{'action'};
+        $href->{'dist'}     ||= $href->{'distversion'};
+        $href->{'grade'}    ||= $href->{'action'} || $href->{'status'};
 
         push @rv, $href;
     }
@@ -317,6 +317,7 @@ sub _send_report {
     my $name    = $mod->module;
     my $dist    = $mod->package_name . '-' . $mod->package_version;
     my $author  = $mod->author->author;
+    my $distfile= $mod->author->cpanid . "/" . $mod->package;
     my $email   = $mod->author->email || CPAN_MAIL_ACCOUNT->( $author );
     my $cp_conf = $conf->get_conf('cpantest') || '';
     my $int_ver = $CPANPLUS::Internals::VERSION;
@@ -358,7 +359,7 @@ sub _send_report {
                 ### version of perl (5.8.6+ and 5.9.2-4 at the time of writing)
                 ### 'Config' is not recognized as a core module. See this bug:
                 ###    http://rt.cpan.org/Ticket/Display.html?id=32155
-                if( not $obj and not $sub->( $prq_name ) ) {
+                if( !$obj and !$sub->( $prq_name ) ) {
                     msg(loc( "Prerequisite '%1' for '%2' could not be obtained".
                              " from CPAN -- sending N/A grade", 
                              $prq_name, $name ), $verbose );
@@ -367,7 +368,20 @@ sub _send_report {
                     last GRADE;        
                 }
 
-                if( $cb->_vcmp( $prq_ver, $obj->installed_version ) > 0 ) {
+                if ( !$obj ) {
+                    my $vcore = $sub->( $prq_name );
+                    if ( $cb->_vcmp( $prq_ver, $vcore ) > 0 ) {
+                      msg(loc( "Version of core module '%1' ('%2') is too low for ".
+                               "'%3' (needs '%4') -- sending N/A grade", 
+                               $prq_name, $vcore, 
+                               $name, $prq_ver ), $verbose );
+                             
+                      $grade = GRADE_NA;
+                      last GRADE;        
+                    }
+                }
+
+                if( $obj and $cb->_vcmp( $prq_ver, $obj->installed_version ) > 0 ) {
                     msg(loc( "Installed version of '%1' ('%2') is too low for ".
                              "'%3' (needs '%4') -- sending N/A grade", 
                              $prq_name, $obj->installed_version, 
@@ -464,6 +478,9 @@ sub _send_report {
         ### add a list of what modules have been loaded of your prereqs list
         $message .= REPORT_LOADED_PREREQS->($mod);
 
+        ### add a list of versions of toolchain modules
+        $message .= REPORT_TOOLCHAIN_VERSIONS->($mod);
+
         ### the footer
         $message .= REPORT_MESSAGE_FOOTER->();
 
@@ -496,6 +513,7 @@ sub _send_report {
         Test::Reporter->new(
             grade           => $grade,
             distribution    => $dist,
+            distfile        => $distfile,
             via             => "CPANPLUS $int_ver",
             timeout         => $conf->get_conf('timeout') || 60,
             debug           => $conf->get_conf('debug'),
@@ -550,13 +568,23 @@ sub _send_report {
         }
 
     ### XXX should we do an 'already sent' check? ###
-    } elsif( $reporter->send( ) ) {
-        msg(loc("Successfully sent '%1' report for '%2'", $grade, $dist),
-            $verbose);
-        return 1;
-
     ### something broke :( ###
-    } else {
+    } 
+    else {
+        my $status;
+        eval { 
+            $status = $reporter->send();
+        };
+        if ( $@ ) {
+           error(loc("Could not send '%1' report for '%2': %3",
+                $grade, $dist, $@));
+           return;
+        }
+        if ( $status ) {
+           msg(loc("Successfully sent '%1' report for '%2'", $grade, $dist),
+              $verbose);
+           return 1;
+        }
         error(loc("Could not send '%1' report for '%2': %3",
                 $grade, $dist, $reporter->errstr));
         return;
